/*
 *  Copyright (C) 2002-25 - ntop.org
 *
 *  http://www.ntop.org/
 *
 * This code is proprietary code subject to the terms and conditions 
 * defined in LICENSE file which is part of this source code package.
 */

#ifndef _ENGINE_H_
#define _ENGINE_H_

/* ********************************** */

#define PCAP_VERY_LONG_SNAPLEN   9000
#define PCAP_LONG_SNAPLEN        1600
#define PCAP_DEFAULT_SNAPLEN      128
#define PCAP_MAX_SNAPLEN       262144 /* tcpdump.c */

#define MAX_NUM_PLUGINS            64
#define MAX_NUM_PLUGIN_L7_PROTOS    3

typedef enum {
  create_flow_callback = 1,
  delete_flow_callback,
  packet_callback,
  l7_protocol_detected_callback
} FlowCallback;
	
struct processFlowPacketInfo {
  int packet_if_idx /* -1 = unknown */;
  u_int8_t rx_packet; /* 1=RX, 0=TX */
  u_int32_t subflow_id;
  u_int8_t proto;
  u_int16_t numFragments;
  u_int16_t ip_offset;
  u_int8_t sampledPacket;
  u_int32_t numPkts;
  u_int8_t tos;
  u_int16_t ip_id;
  u_int8_t ttl;
  u_int16_t vlanId_src;
  u_int16_t vlanId_dst;
  u_int16_t innerVlanId;
  u_int16_t outerVlanId;
  u_int32_t tunnel_id;
  u_int16_t session_id;
  u_int16_t gtp_offset;
  struct eth_header *ehdr;
  IpAddress *src;
  u_int16_t sport;
  IpAddress *dst;
  u_int16_t dport;
  u_int8_t untunneled_proto;
  IpAddress *untunneled_src;
  u_int16_t untunneled_sport;
  IpAddress *untunneled_dst;
  u_int16_t untunneled_dport;
  u_int16_t ndpi_proto_major;
  u_int len;
  u_int16_t tcpWin;
  u_int8_t tcpFlags;
  u_int32_t tcpSeqNum;
  u_int32_t tcpAckNum;
  u_int16_t tcpLen;
  u_int16_t tcpMaxSegmentSize;
  u_int8_t tcpWinScale;
  u_int8_t icmpType;
  u_int8_t icmpCode;
  u_int16_t numMplsLabels;
  u_char *mplsLabels;
  u_int32_t if_input;
  u_int32_t if_output;
  struct pcap_pkthdr *h;
  u_char *p;
  u_char *tcp_udp;
  u_char *payload;
  u_int16_t payload_shift;
  u_int payloadLen;
  u_int originalPayloadLen;
  const struct timeval *_firstSeen; /* Always set to 0 unless numPkts > 0 */
  as_info as;
  u_int16_t src_mask;
  u_int16_t dst_mask;
  u_int32_t flow_sender_ip;
  struct in6_addr *flow_sender_ip6;
  u_int32_t packet_hash;
  u_int8_t engine_type;
  u_int8_t engine_id;
  char *osi_src;
  char *osi_dst;
    struct {
      char *ssid;
      u_int8_t *access_point_mac;
  } wifi;
  flow_marker *verdict;
  flow_source source_nature;
  struct generic_netflow_record *record;  
};

extern int checkPluginExport(u_int elementIdx, V9V10TemplateElementId *theTemplate, FlowDirection direction,
			     FlowHashBucket *theFlow, char *outBuffer,
			     uint* outBufferBegin, uint* outBufferMax);
extern int checkPluginPrint(V9V10TemplateElementId *theTemplate, FlowDirection direction,
			    FlowHashBucket *bkt, char *line_buffer, uint line_buffer_len,
			    u_int8_t json_mode);
typedef V9V10TemplateElementId* (*PluginConf)(void);
extern void discardBucket(FlowHashBucket *myBucket);
extern void* dequeueBucketToExport(void*);
typedef void (*PluginInitFctn)(void);
typedef void (*PluginTermFctn)(void);
typedef void (*PluginFctn)(FlowHashBucket*, void*);
typedef void (*PluginPacketFctn)(FlowCallback callback, 
				 int packet_if_idx /* -1 = unknown */,
				 PluginInformation **pluginInfo,
				 FlowHashBucket *bkt,
				 FlowDirection flow_direction,
				 u_int16_t ip_offset, u_int16_t proto, u_char isFragment,
				 u_int16_t numPkts, u_char tos,
				 u_int8_t retransmitted_pkt,
				 u_int16_t vlanId, struct eth_header *ehdr,
				 IpAddress *src, u_int16_t sport,
				 IpAddress *dst, u_int16_t dport,
				 u_int len, u_int8_t flags, u_int32_t tcpSeqNum,
				 u_int8_t icmpType, u_int16_t numMplsLabels,
				 u_char *mplsLabels /* [MAX_NUM_MPLS_LABELS][MPLS_LABEL_LEN] */,
				 const struct pcap_pkthdr *h, const u_char *p,
				 u_char *udp_tcp, /* Pointer to UDP/TCP */
				 u_char *payload, int payloadLen,
				 struct processFlowPacketInfo *pfinfo);
typedef V9V10TemplateElementId* (*PluginGetTemplateFctn)(char* template_name);
typedef int (*PluginExportFctn)(void*, u_int elementIdx, V9V10TemplateElementId *theTemplate, FlowDirection direction,
				FlowHashBucket *theFlow, char *outBuffer,
				uint* outBufferBegin, uint* outBufferMax);
typedef int (*PluginPrintFctn)(void*, V9V10TemplateElementId *theTemplate, FlowDirection direction,
			       FlowHashBucket *theFlow, char *line_buffer, uint line_buffer_len,
			       u_int8_t json_mode, u_int8_t *is_string);
typedef const struct option* (*PluginOptionsFctn)(void);
typedef void (*PluginStatsFctn)(void);
typedef void (*PluginSetupFctn)(void);
typedef void (*PluginHelpFctn)(void);
typedef void (*PluginIdleTaskFctn)(void);
typedef void (*PluginIdleFlowFctn)(FlowHashBucket *theFlow, void *pluginData);
typedef char* (*PluginGetTemplateShortcut)(char *shortcut);

typedef struct pluginEntryPoint {
  char *nprobe_revision, *name, *short_name, *family, *pro_family, *version, *descr, *author;
  u_int8_t skip_in_collector_mode;
  u_int8_t always_enabled, enabled, need_license, pro_version;
  PluginInitFctn initFctn;
  PluginOptionsFctn optionsFctn;
  PluginTermFctn termFctn;
  PluginConf pluginFlowConf;
  PluginFctn deleteFlowFctn;
  u_char call_packetFlowFctn_for_each_packet;
  PluginPacketFctn packetFlowFctn;
  PluginGetTemplateFctn getTemplateFctn;
  PluginExportFctn pluginExportFctn;
  PluginPrintFctn pluginPrintFctn;
  PluginStatsFctn pluginStatsFctn;
  PluginSetupFctn setupFctn;
  PluginHelpFctn helpFctn;
  PluginIdleTaskFctn idleFctn;
  PluginIdleFlowFctn idleFlowFctn;
  PluginGetTemplateShortcut templateShortcut;
  u_int8_t v4TemplateIdx, v6TemplateIdx; /* 
					    Template indexes for this plugin 
					    on readOnlyGlobals.templateBuffers[XXX] 
					 */
} PluginEntryPoint;

extern PluginEntryPoint* PluginEntryFctn(void);

#define PLUGIN_DONT_NEED_LICENSE   0
#define PLUGIN_NEED_LICENSE        1

#define PLUGIN_STANDARD_VERSION    0
#define PLUGIN_PRO_VERSION         1

/* ********************************** */

extern char* _intoa(IpAddress addr, char* buf, u_int16_t bufLen);
extern char* _intoaV4(unsigned int addr, char* buf, u_int16_t bufLen);
extern char* _intoaV6(struct in6_addr *addr, char* buf, u_int16_t bufLen);
extern char* formatTraffic(float numBits, int bits, char *buf);
extern char* formatPackets(float numPkts, char *buf);
extern u_char ttlPredictor(u_char x);
extern char* proto2name(u_int8_t proto);
extern void load_mappings(void);
extern void unload_mappings(void);
extern u_int8_t setPayload(FlowHashBucket *bkt, const struct pcap_pkthdr *h,
			   u_char *p, u_int16_t ip_offset, u_char *payload, int payloadLen, 
			   FlowDirection direction);
extern void setnDPIProto(FlowHashBucket *bkt, ndpi_protocol proto_id);
extern u_int64_t getLRUCacheKey(FlowHashBucket *bkt);
extern int cmpIpAddress(IpAddress *src, IpAddress *dst);
extern void printICMPflags(u_int8_t proto, u_int32_t flags, char *icmpBuf, int icmpBufLen);
extern void printFlow(FlowHashBucket *theFlow, FlowDirection direction);
extern void printBucket(FlowHashBucket *myBucket);
extern void walkHash(u_int8_t flushHash);
extern void purgeBucket(FlowHashBucket *myBucket);
extern void processGTPFlowPacket(u_int32_t gtp_teid,
				 struct pcap_pkthdr *h, u_int gtp_pkt_len);
extern FlowHashBucket* processFlowPacket(struct processFlowPacketInfo *pinfo);

extern struct timeval* getFlowBeginTime(FlowHashBucket *theFlow, FlowDirection direction);
extern struct timeval* getFlowEndTime(FlowHashBucket *theFlow, FlowDirection direction);
extern u_int32_t getFlowDurationSec(FlowHashBucket *theFlow);
extern V9V10TemplateElementId* getPluginTemplate(char* template_name, PluginEntryPoint **plugin);
extern char* expandPluginTemplate(char** template_name);
extern void updateCollectedFlowThpt(FlowDirection direction, FlowHashBucket *theFlow);
extern void updateFlowThpt(struct timeval *stamp, FlowDirection direction, FlowHashBucket *theFlow);

/* nprobe.c or nprobe_mod.c */
extern void queueBucketToExport(FlowHashBucket *myBucket);

/* plugin.c */
extern u_int16_t num_plugins_enabled;
extern void initPlugins(void);
extern PluginEntryPoint* get_plugin_info(char *short_name);
extern void loadPlugins(void);
extern void setPluginDir(char *dirName);
extern const struct option* buildCLIOptions(void);
extern void termPlugins(void);
extern void pluginCallback(FlowCallback callbackType, 
			   int packet_if_idx /* -1 = unknown */,
			   FlowHashBucket* bucket,
			   FlowDirection direction,
			   u_int16_t ip_offset, u_int16_t proto, u_char isFragment,
			   u_int16_t numPkts, u_char tos,
			   u_int8_t retransmitted_pkt,
			   u_int16_t vlanId, /* We pass the srcVlanId (for collect flows srcVlanId might be != dstVlanId) */
			   struct eth_header *ehdr,
			   IpAddress *src, u_int16_t sport,
			   IpAddress *dst, u_int16_t dport,
			   u_int len, u_int8_t flags, u_int32_t tcpSeqNum, 
			   u_int8_t icmpType, u_int16_t numMplsLabels,
			   u_char *mplsLabels,
			   const struct pcap_pkthdr *h, const u_char *p,
			   u_char *udp_tcp, /* Pointer to UDP/TCP */
			   u_char *payload, int payloadLen,
			   struct processFlowPacketInfo *pfinfo);
extern void buildActivePluginsList(V9V10TemplateElementId *template_element_list[]);
extern void printMetadata(FILE *file);
extern void pluginIdleThreadTask(void);

/* engine.c */
extern void checkExportFileClose(void);
extern void check_dump_file_open(u_int do_lock);
extern u_int32_t get_flow_serial(void);
extern void ndpiFlowGiveup(FlowHashBucket *bkt);
extern void computeEntropy(struct entropy_stats_direction *entropy);
extern void allocateFlowHash();

/* qoe.c */
extern void updateTCPAck(FlowHashBucket *bkt, struct timeval *when,
			 FlowDirection direction, u_int32_t ack_id);
extern u_int8_t getQoEScore(FlowHashBucket *bkt);
extern void updateQUICStats(FlowHashBucket *bkt,
			    FlowDirection direction, struct timeval *tv,
			    u_int8_t *payload, u_int16_t payload_len);

#endif /* _ENGINE_H_ */
